Package: ExiMiR
Type: Package
Title: R functions for the normalization of Exiqon miRNA array data
Version: 2.55.0
Date: 2012-11-06
Author: Sylvain Gubian <DL.RSupport@pmi.com>, Alain Sewer
        <DL.RSupport@pmi.com>, PMP SA
Maintainer: Sylvain Gubian <DL.RSupport@pmi.com>
Depends: R (>= 2.10), Biobase (>= 2.5.5), affy (>= 1.26.1), limma
Imports: affyio(>= 1.13.3), Biobase(>= 2.5.5), preprocessCore(>=
        1.10.0)
Suggests: mirna10cdf
Description: This package contains functions for reading raw data in
        ImaGene TXT format obtained from Exiqon miRCURY LNA arrays,
        annotating them with appropriate GAL files, and normalizing
        them using a spike-in probe-based method. Other platforms and
        data formats are also supported.
License: GPL-2
Collate: make.gal.env.R read.exi.R createAB.R NormiR.R NormiR.methods.R
LazyLoad: yes
biocViews: Microarray, OneChannel, TwoChannel, Preprocessing,
        GeneExpression, Transcription
git_url: https://git.bioconductor.org/packages/ExiMiR
git_branch: devel
git_last_commit: 271382b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:35:41 UTC; biocbuild
